/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.webcommon;

/** Abstract class an instance of which is used to signal open and close
 * events to the web application.
 *
 * @author Mike Douglass douglm@rpi.edu
 */
public abstract class UWCalCallback {
  /** This callback should be installed in the session as an attribute with
   this name. Then either a filter or request listener (but not both) will
   be able to use the callback to open/close the backend.
   */
  public static final String cbAttrName =
      "edu.rpi.cct.uwcal.webcommon.UWCalSvciFilter";

  /** Called when the response is on its way in.
   */
  public abstract void in() throws Throwable;

  /** Called when the response is on its way out.
   */
  public abstract void out() throws Throwable;

  /** Called after the above to allow close etc.
   */
  public abstract void close() throws Throwable;
}


