// java

package edu.washington.cac.calendar.filter;

import org.apache.regexp.RE;

import edu.washington.cac.calendar.data.Event;
import edu.washington.cac.calendar.data.PublicEvents;

/**
  A filter that selects events that match a regular expression in
  the location's address field

  @author Greg Barnes
  @version 1.0
 */
public class RegexpLAddressFilter extends RegexpFilter
{
  /**
    Create a new instance
    @param re Regular expression to match
   */
  RegexpLAddressFilter(RE re)
  {
    super(re);
  }

  /** 
    Does the filter selects the event?
    @param e Event to test
    @return If the filter selects the event
   */
  public boolean match(Event e)
  {
    return getRe().match(
        e.getLocation(PublicEvents.getPublicLocations()).getAddress());
  }

  /**
    Get a human-readable name for the calendar represented by the filter
    @return a human-readable name for the calendar represented by the filter
   */
  public String getName() 
  {
    return "Events whose address matches " + getRe();
  }
}
