package edu.washington.cac.calenv;

import java.sql.Connection;
import java.util.Properties;
import javax.naming.InitialContext;
import javax.sql.DataSource;

/** An implementation of CalEnv which obtains all the required properties
 * from a properties resource file but uses a DataSource.
 */
public class FileDSProperties extends CalEnv {
  private boolean debug;

  /* ====================================================================
   *              Implementations of the abstract methods
   * ==================================================================== */

  /** Called after we obtain the object.
   *
   * @param debug     boolean true for debugging
   */
  public void init(boolean debug) throws CalEnvException {
    /* Nothing more to do here - we just take the properties fom the
       supercalsses resource file. */
    this.debug = debug;
  }

  /** Returns a Connection object for database interactions.
   *
   * @return Connection  to the database.
   */
  public Connection getConnection() throws CalEnvException {
    /** This gets a resource from the container.
     * In the web.xml file you need something like:
     *  <!-- JDBC DataSources (java:comp/env/jdbc) -->
     *  <resource-ref>
     *    <description>The Calendar DS</description>
     *    <res-ref-name>jdbc/CalDB</res-ref-name>
     *    <res-type>javax.sql.DataSource</res-type>
     *    <res-auth>Container</res-auth>
     *  </resource-ref>
     *
     */

    try {
      InitialContext ctx = new InitialContext();

      DataSource ds = (DataSource) ctx.lookup("java:comp/env/jdbc/calDB");


      return ds.getConnection();
    } catch (Throwable t) {
      t.printStackTrace();
      throw new CalEnvException(t.getMessage());
    }
  }
}

