package edu.washington.cac.calfacade.shared;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

/** An object representing a calendar admin group.
 *
 * @author Mike Douglass douglm@rpi.edu
 * @version 2.2
 */
public class AdminGroupVO extends AdminGroupROVO {
  /** Create a new object.
   */
  public AdminGroupVO() {
  }

  /** Create a new object.
   *
   * @param  name          String group name
   * @param  description   String group description
   * @param  groupOwner    UserVO user who owns the group.
   * @param  groupMembers   Collection of UserVO
   */
  public AdminGroupVO(String name,
                       String description,
                       UserVO groupOwner,
                       UserVO owner,
                       Collection groupMembers) {
    super(name, description, groupOwner, owner, groupMembers);
  }

  /** Set the name of the group.
   *
   * @param   val     String group name.
   * @exception CalFacadeException If there's a problem
   */
  public void setName(String val) throws CalFacadeException {
    name = val;
  }

  /** Set the description of the group.
   *
   * @param   val     String group description.
   * @exception CalFacadeException If there's a problem
   */
  public void setDescription(String val) throws CalFacadeException {
    description = val;
  }

  /** Set the id of the group owner.
   *
   * @param   val     UserVO group owner.
   * @exception CalFacadeException If there's a problem
   */
  public void setGroupOwner(UserVO val) throws CalFacadeException {
    groupOwner = val;
  }

  /** Set the ownerid of the group.
   *
   * @param   val     UserVO group event owner.
   * @exception CalFacadeException If there's a problem
   */
  public void setOwner(UserVO val) throws CalFacadeException {
    owner = val;
  }

  /** Set the members of the group.
   *
   * @param   val     Collection of group members.
   * @exception CalFacadeException If there's a problem
   */
  public void setGroupMembers(Collection val) {
    groupMembers = val;
  }

  /** get a read-only version
   *
   * @return AdminGroupROVO   read only version of this object
   */
  public AdminGroupROVO getAdminGroupROVO() {
    return new AdminGroupROVO(name, description, groupOwner,
                              owner, groupMembers);
  }

  /* ====================================================================
   *                   Convenience methods
   * ==================================================================== */

  /** Add a group member. Return true if is was added, false if it was in
   * the list
   *
   * @param u          UserVO to add
   * @return boolean   true if added
   */
  public boolean addGroupMember(UserVO u) {
    if (groupMembers == null) {
      groupMembers = new Vector();
    } else if (groupMembers.contains(u)) {
      return false;
    }

    groupMembers.add(u);
    return true;
  }

  /** Remove a group member. Return true if is was removed, false if it was
   * not in the list
   *
   * @param u          UserVO to remove
   * @return boolean   true if removed
   */
  public boolean removeGroupMember(UserVO u) {
    if (groupMembers == null) {
      return false;
    }

    if (!groupMembers.contains(u)) {
      return false;
    }

    groupMembers.remove(u);
    return true;
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("AdminGroupVO{name=");
    sb.append(name);
    sb.append(", description=");
    sb.append(description);
    sb.append(", groupOwner=");

    if (groupOwner == null) {
      sb.append("**********null************");
    } else {
      sb.append(groupOwner.getAccount());
    }
    sb.append(", owner=");

    if (owner == null) {
      sb.append("**********null************");
    } else {
      sb.append(owner.getAccount());
    }
    sb.append(", groupMembers={");
    boolean first = true;
    if (groupMembers != null) {
      Iterator it = groupMembers.iterator();
      while (it.hasNext()) {
        UserVO u = (UserVO)it.next();
        if (!first) {
          sb.append(", ");
        }
        first = false;
        sb.append(u.getId());
      }
    }
    sb.append("}");
    sb.append("}");

    return sb.toString();
  }

  public Object clone() {
    try {
      Collection c = getGroupMembers();

      Vector v = new Vector();

      if (c != null) {
        Iterator it = c.iterator();

        while (it.hasNext()) {
          UserVO u = (UserVO)it.next();
          v.add((UserVO)u.clone());
        }
      }

      return new AdminGroupVO(getName(),
                              getDescription(),
                              (UserVO)getGroupOwner().clone(),
                              (UserVO)getOwner().clone(),
                              v);
    } catch (CalFacadeException cfe) {
      throw new RuntimeException(cfe);
    }
  }
}
