package edu.washington.cac.calfacade.shared;

import java.io.Serializable;

/** Represents a selection of a public event by a particular user.
 *
 *  @author Mike Douglass douglm@rpi.edu
 *  @version 1.0
 */
public class EventRefVO implements CalFacadeDefs, Serializable {
  /** Internal db id of the event */
  private int eventid;

  private int userid = unsavedItemKey;

  /** The event */
  private EventVO event;

  /** The user */
  private UserVO user;

  /** the default purpose ('E' for 'explicitly added') */
  private static final char DefaultPurpose = 'E';

  /** A one-character code for the 'purpose' of the eventref */
  private char purpose = DefaultPurpose;

  public EventRefVO() {
  }

  /** Constructor setting all fields.
   *
   * @param event      EventVO object for which we create the eventref
   * @param user       UserVO user for which we create the eventref
   * @param purpose    A simple code explaining why this eventref exists
   */
  public EventRefVO(EventVO event, UserVO user, char purpose) {
    setEvent(event);
    setUser(user);
    this.purpose = purpose;
  }

  /** Create an EventRefVO with defaulted purpose
   *
   * @param event      EventVO object for which we create the eventref
   * @param user       UserVO user for which we create the eventref
   */
  public EventRefVO(EventVO event, UserVO user) {
    this(event, user, DefaultPurpose);
  }

  /** set the eventid
   *
   * @param val  int event id.
   */
  public void setEventid(int val) {
    eventid = val;
  }

  /** Get the eventid
   *
   * @return int     event id.
   */
  public int getEventid() {
    return eventid;
  }

  /**    Set the userid
   * @param val   userid
   */
  public void setUserid(int val) {
    userid = val;
  }

  /** Get the userid
   *
   * @return int    the userid
   */
  public int getUserid() {
    return userid;
  }

  /** set the event
   *
   * @param val  event user
   */
  public void setEvent(EventVO val) {
    event = val;
    if (event != null) {
      eventid = val.getId();
    } else {
      eventid = unsavedItemKey;
    }
  }

  /** Get the event
   *
   * @return eventid     user
   */
  public EventVO getEvent() {
    return event;
  }

  /** set the user
   *
   * @param val  UserVO user
   */
  public void setUser(UserVO val) {
    user = val;
    if (user != null) {
      userid = val.getId();
    } else {
      userid = unsavedItemKey;
    }
  }

  /** Get the user
   *
   * @return UserVO     user
   */
  public UserVO getUser() {
    return user;
  }

  /**  set the purpose of this reference
   *
   * @param val    char purpose of this reference
   */
  public void setPurpose(char val) {
    purpose = val;
  }

  /**  Get the purpose of this reference
   *
   * @return char    purpose of this reference
   */
  public char getPurpose() {
    return purpose;
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public int hashCode() {
    return eventid * userid;
  }

  public boolean equals(Object obj) {
    if (!(obj instanceof EventRefVO)) {
      return false;
    }

    EventRefVO er = (EventRefVO)obj;

    return (er.eventid == eventid) && (er.userid == userid);
  }
}
