package edu.washington.cac.calfacade.shared;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.Vector;

/**
 * Recurrence information for an event.
 *
 * <p>The information consists of the following components<ul>
 * <li>Zero or more exclusion date/times</li>
 * <li>Zero or more exception rules</li>
 * <li>Zero or more recurrence date/times</li>
 * <li>One or more recurrence rules</li>
 *
 * @author Mike Douglass douglm@rpi.edu
 * @version 1.0
 */
public class RecurVO implements Serializable {
  /** Associated event */
  private EventVO event;

  /** Collection of Date */
  private Vector exclusionDates;

  /** Collection of RecurRuleVO */
  private Vector exclusionRules;

  /** Collection of Date */
  private Vector recurrenceDates;

  /** Collection of RecurRuleVO */
  private Vector recurrenceRules;

  /** True if we expanded all occurrences.
   */
  private boolean expanded;
  /** Constructor
   *
   */
  public RecurVO() {
  }

  /** Constructor
   *
   */
  public RecurVO(EventVO event,
                 Collection exclusionDates,
                 Collection exclusionRules,
                 Collection recurrenceDates,
                 Collection recurrenceRules,
                 boolean expanded) {
    this.event = event;
    this.exclusionDates = (Vector)exclusionDates;
    this.exclusionRules = (Vector)exclusionRules;
    this.recurrenceDates = (Vector)recurrenceDates;
    this.recurrenceRules = (Vector)recurrenceRules;
    this.expanded = expanded;
  }

  public void setEvent(EventVO val) {
    event = val;
  }

  public EventVO getEvent() {
    return event;
  }

  public void setExclusionDates(Collection val) {
    exclusionDates = (Vector)val;
  }

  public Collection getExclusionDates() {
    if (exclusionDates == null) {
      exclusionDates = new Vector();
    }

    return exclusionDates;
  }

  public void setExclusionRules(Collection val) {
    exclusionRules = (Vector)val;
  }

  public Collection getExclusionRules() {
    if (exclusionRules == null) {
      exclusionRules = new Vector();
    }

    return exclusionRules;
  }

  public void setRecurrenceDates(Collection val) {
    recurrenceDates = (Vector)val;
  }

  public Collection getRecurrenceDates() {
    if (recurrenceDates == null) {
      recurrenceDates = new Vector();
    }

    return recurrenceDates;
  }

  public void setRecurrenceRules(Collection val) {
    recurrenceRules = (Vector)val;
  }

  public Collection getRecurrenceRules() {
    if (recurrenceRules == null) {
      recurrenceRules = new Vector();
    }

    return recurrenceRules;
  }

  public void setExpanded(boolean val) {
    expanded = val;
  }

  public boolean getExpanded() {
    return expanded;
  }

  /** ===================================================================
   *                   Convenience methods
   *  =================================================================== */

  public void addExclusionDate(Date val) {
    Collection c = getExclusionDates();

    if (!c.contains(val)) {
      c.add(val);
    }
  }

  public void addExclusionRule(RecurRuleVO val) {
    Collection c = getExclusionRules();

    if (!c.contains(val)) {
      c.add(val);
    }
  }

  public void addRecurrenceDate(Date val) {
    Collection c = getRecurrenceDates();

    if (!c.contains(val)) {
      c.add(val);
    }
  }

  public void addRecurrenceRule(RecurRuleVO val) {
    Collection c = getRecurrenceRules();

    if (!c.contains(val)) {
      c.add(val);
    }
  }

  /** ===================================================================
   *                   Object methods
   *  =================================================================== */

  public Object clone() {
    return new RecurVO((EventVO)getEvent().clone(),
                       (Collection)((Vector)getExclusionDates()).clone(),
                       (Collection)((Vector)getExclusionRules()).clone(),
                       (Collection)((Vector)getRecurrenceDates()).clone(),
                       (Collection)((Vector)getRecurrenceRules()).clone(),
                       expanded);
  }
}
