package edu.washington.cac.calfacade.shared;

import java.io.Serializable;
import java.sql.Timestamp;

/** Information about synchronization with a given device
 *
 *  @author Mike Douglass douglm@rpi.edu
 *  @version 1.0
 */
public class SynchInfoVO implements Serializable {
  private int userid;

  /** The user */
  private UserVO user;

  /** The deviceId identifies the other end of the conversation. This is
   * something of an Syncml term
   */
  private String deviceId;

  /** The calendarId determines what view of the users data we are synching
   * with. For example, for device "MyPalm" we might only be synching movies.
   *
   * <p>For the time being at least we are only synching the entire visible
   * set of user events - defined by a calendarId of < 0
   */
  private int calendarId = -1;

  /** The event this refers to. If the event has been deleted this will be
   * null, however the guid will be set and can be used by the synch engine.
   */
  private Timestamp lastsynch;

  public SynchInfoVO() {
  }

  public SynchInfoVO(UserVO user,
                     String deviceId,
                     int calendarId,
                     Timestamp lastsynch) {
    this.user = user;
    if (user != null) {
      this.userid = user.getId();
    }
    this.deviceId = deviceId;
    this.calendarId = calendarId;
    this.lastsynch = lastsynch;
  }

  /**    Set the userid
   * @param val   userid
   */
  public void setUserid(int val) {
    userid = val;
  }

  /** Get the unique id
   *
   * @return int    the userid
   */
  public int getUserid() {
    return userid;
  }

  /** set the user
   *
   * @param val  UserVO user
   */
  public void setUser(UserVO val) {
    user = val;
    if (user != null) {
      userid = user.getId();
    }
  }

  /** Get the user
   *
   * @return UserVO     user, userVO
   */
  public UserVO getUser() {
    return user;
  }

  /** set the deviceId
   *
   * @param val  String deviceId
   */
  public void setDeviceId(String val) {
    deviceId = val;
  }

  /** Get the deviceId
   *
   * @return String     deviceId
   */
  public String getDeviceId() {
    return deviceId;
  }

  /** set the calendarId
   *
   * @param val  int calendarId
   */
  public void setCalendarId(int val) {
    calendarId = val;
  }

  /** Get the calendarId
   *
   * @return int     calendarId
   */
  public int getCalendarId() {
    return calendarId;
  }

  public void setLastsynch(Timestamp val) {
    lastsynch = val;
  }

  public Timestamp getLastsynch() {
    return lastsynch;
  }
}
