package edu.washington.cac.calfacade.shared;

import java.util.Collection;
import java.util.Comparator;
import java.util.Set;

/** Value object to represent a calendar user.
 *
 *   @author Mike Douglass douglm@rpi.edu
 *  @version 1.0
 */
public class UserVO extends PrincipalVO {
  /* Admin groups this user is a member of */
  protected Set adminGroups;

  /* ====================================================================
   *                   Constructors
   * ==================================================================== */

  /** Create a guest user vo
   */
  public UserVO() {
    super();
  }

  /** Create a user vo usually based on a single string name from an object,
   * e.g., an event.
   *
   * @param  name            String user id for database
   */
  public UserVO(String name) {
    super(name);
  }

  /** Create a user vo by specifying all its fields
   *
   * @param  account         String user account for database
    */
  public UserVO(String account,
                int id,
                int rights,
                boolean publicEvents) {
    super(account, id, rights, publicEvents);
  }

  /** Set of admin groups of which user is a member
   *
   * @param val        Set of AdminGroupVO
   */
  public void setAdminGroups(Set val) {
    adminGroups = val;
  }

  /** Get the admin groups of which user is a member.
   *
   * @return Set    of CalendarVO
   */
  public Set getAdminGroups() {
    return adminGroups;
  }

  /* ====================================================================
   *                   Copying methods
   * ==================================================================== */

  public void copyTo(UserVO val) {
    super.copyTo(val);
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public Object clone() {
    /* We do not clone the attached subscriptions if present. These need to
       be cloned explicitly or we might set up a clone loop.
    */
    UserVO u = new UserVO();
    copyTo(u);

    return u;
  }
}
