// java

package edu.washington.cac.calfacade.shared.filter;

import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.KeywordVO;
import edu.washington.cac.calfacade.shared.UserVO;

/**
  A filter that maps on to another filter

  @author Greg Barnes
  @version 1.0
 */
public class AliasFilterVO extends FilterVO
{
  /** The target filter */
  protected FilterVO target;

  /** Get a human-readable name for the calendar represented by the filter
   *
   * @return String  a human-readable name for the filter
   */
  public String getTitle() {
    if (title != null) {
      return title;
    }

    return target.getTitle();
  }

  /** Set the target filter
   *
   *  @param val     FilterVO target filter
   */
  public void setTarget(FilterVO val) {
    target = val;
  }

  /** Get the target filter
   *  @return FilterVO     target filter
   */
  public FilterVO getTarget() {
    return target;
  }

  /** Test if this event passes the test
   *
   * @param             e Event to test
   * @return boolean    true if the filter selects the event
   */
  public boolean match(EventVO e) {
    return target.match(e);
  }

  /* ====================================================================
   *                   Copying methods
   * ==================================================================== */

  protected void copyTo(AliasFilterVO val) {
    super.copyTo(val);
    val.setTarget((FilterVO)getTarget().clone());
  }

  /* ====================================================================
   *                   Object methods
   * ==================================================================== */

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("AliasFilterVO{id=");
    sb.append(id);
    sb.append(", name=");
    sb.append(getName());
    sb.append(", target=");
    sb.append(target);
    sb.append("}");

    return sb.toString();
  }

  public Object clone() {
    AliasFilterVO f = new AliasFilterVO();

    copyTo(f);

    return f;
  }
}
