/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.tests.rpi.cct.uwcal.synch.common;

import edu.tests.rpi.cct.uwcal.calsvci.CalSvcTestUtil;

import edu.rpi.cct.uwcal.common.URIgen;
import edu.rpi.cct.uwcal.synch.common.IcalSynchState;
import edu.rpi.cct.uwcal.synch.common.Synch;
import edu.rpi.cct.uwcal.webcommon.UWCalWebURIgen;

import edu.washington.cac.calfacade.shared.CalFacadeDefs;
import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.UserVO;

import java.util.Collection;
import java.util.Iterator;
import junit.framework.TestCase;

/** Test the Synch class
 *
 * @author Mike Douglass       douglm@rpi.edu
 */
public class SynchTest extends TestCase {
  private Synch synch;

  private boolean debug = true;

  private static String user = "caluser";

  private static String deviceId = "MyPalm";

  public void testSynch() {
    try {
      URIgen urigen = new UWCalWebURIgen("http://cal.rpi.edu");

      synch = new Synch(user, deviceId, urigen, debug);

      String guid = synch.getNewGuid();

      log("Obtained new guid: " + guid);

      EventVO ev = synch.findEvent(guid);

      log("Found event: " + ev);

      Collection es = synch.getAllVevents();
      Iterator it = es.iterator();

      while (it.hasNext()) {
        IcalSynchState iss = (IcalSynchState)it.next();

        log("got synchstate for guid: " + iss.getGuid() +
            " state: " + iss.getSynchState());
      }
    } catch (Throwable t) {
      t.printStackTrace();
      fail("Exception: " + t.getMessage());
    }
  }

  /* ====================================================================
   *                       Private methods.
   * ==================================================================== */

  private void log(String msg) {
    System.out.println(this.getClass().getName() + ": " + msg);
  }
}

