/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.tests.rpi.cct.uwcal.webcommon;

import edu.rpi.cct.uwcal.common.URIgen;
import edu.rpi.cct.uwcal.webcommon.UWCalWebURIgen;

import edu.washington.cac.calfacade.shared.LocationVO;
import edu.washington.cac.calfacade.shared.SponsorVO;

import java.net.URI;

import junit.framework.TestCase;

/** Test the UWCalWebURIgen class
 *
 * @author Mike Douglass       douglm@rpi.edu
 */
public class UWCalWebURIgenTest extends TestCase {
  public void testUWCalWebURIgen() {
    try {
      LocationVO loc = new LocationVO();
      loc.setId(12345);

      SponsorVO sp = new SponsorVO();
      sp.setId(67890);

      URIgen urigen = new UWCalWebURIgen("http://cal.rpi.edu");

      URI locUri = urigen.getLocationURI(loc);

      System.out.println("locuri=" + locUri);

      URI spUri = urigen.getSponsorURI(sp);

      System.out.println("spUri=" + spUri);

      LocationVO loc2 = urigen.getLocation(locUri);
      SponsorVO sp2 = urigen.getSponsor(spUri);

      assertNotNull("Location", loc2);
      assertTrue("Location matches", loc2.getId() == loc.getId());

      assertNotNull("Sponsor", sp2);
      assertTrue("Sponsor matches", sp2.getId() == sp.getId());
    } catch (Throwable t) {
      t.printStackTrace();
      fail("Exception testing URIgen: " + t.getMessage());
    }
  }

  /* ====================================================================
   *                       Private methods.
   * ==================================================================== */

  private void log(String msg) {
    System.out.println(this.getClass().getName() + ": " + msg);
  }
}

