package edu.rpi.cct.uwcal.tools.dump;

import edu.washington.cac.calfacade.shared.AdminGroupVO;
import edu.washington.cac.calfacade.shared.AuthUserVO;
import edu.washington.cac.calfacade.shared.AuthUserPrefsVO;
import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.EventRefVO;
import edu.washington.cac.calfacade.shared.filter.AliasFilterVO;
import edu.washington.cac.calfacade.shared.filter.AndFilterVO;
import edu.washington.cac.calfacade.shared.filter.CreatorFilterVO;
import edu.washington.cac.calfacade.shared.filter.FilterVO;
import edu.washington.cac.calfacade.shared.filter.KeywordFilterVO;
import edu.washington.cac.calfacade.shared.filter.LocationFilterVO;
import edu.washington.cac.calfacade.shared.filter.NotFilterVO;
import edu.washington.cac.calfacade.shared.filter.OrFilterVO;
import edu.washington.cac.calfacade.shared.filter.SponsorFilterVO;
import edu.washington.cac.calfacade.shared.KeywordAttrVO;
import edu.washington.cac.calfacade.shared.KeywordVO;
import edu.washington.cac.calfacade.shared.LocationVO;
import edu.washington.cac.calfacade.shared.SponsorVO;
import edu.washington.cac.calfacade.shared.UserAuth;
import edu.washington.cac.calfacade.shared.UserVO;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Iterator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;

/** Class which implements the dataVO functions needed to dump the
 * calendar dataVO using a jdbc connection.
 *
 * @author Mike Douglass   douglm@rpi.edu
 * @version 1.0
 */
public class JdbcDump implements DumpIntf, JdbcDefs {
  private Connection conn;
  private String url;
  private String className;
  private String id;
  private String pw;
  private int dbType;

  private static final int gettingFilters = 0;
  private static final int gettingUsers = 1;
  private static final int gettingLocations = 2;
  private static final int gettingSponsors = 3;
  private static final int gettingKeywords = 4;
  private static final int gettingEvents = 5;
  private static final int gettingEventRefs = 6;
  private static final int gettingAuthUsers = 7;
  private static final int gettingAdminGroups = 8;
  private static final int gettingDbLastmods = 9;

  /* recurrence stuff */

  private HashMap filters = new HashMap();
  private HashMap usersById = new HashMap();
  private HashMap usersByAccount = new HashMap();
  private HashMap locations = new HashMap();
  private HashMap sponsors = new HashMap();
  private HashMap keywords = new HashMap();
  private HashMap events = new HashMap();

  public JdbcDump(String url,
                  String className,
                  String id,
                  String pw,
                  int dbType) throws Throwable {
    this.url = url;
    this.className = className;
    this.id = id;
    this.pw = pw;
    this.dbType = dbType;
  }

  public void open() throws Throwable {
    Class c = Class.forName(className);
    if (c == null) {
      throw new Exception("Unable to load class " + className);
    }

    conn = DriverManager.getConnection(url, id, pw);
  }

  public void close() {
    try {
      conn.close();
    } catch (Throwable t) {
    }
  }

  /** Will return an Iterator returning FilterVO objects.
   */
  public Iterator getFilters() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getFiltersSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingFilters;

    return it;
  }

  /** Return an Iterator returning UserVO objects
   */
  public Iterator getUsers() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getUsersSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingUsers;

    return it;
  }

  /** Will return an Iterator returning LocationVO objects.
   */
  public Iterator getLocations() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getLocationsSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingLocations;

    return it;
  }

  /** Will return an Iterator returning SponsorVO objects.
   */
  public Iterator getSponsors() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getSponsorsSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingSponsors;

    return it;
  }

  /** Will return an Iterator returning KeywordVO objects.
   * Attributes will be attached
   */
  public Iterator getKeywords() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getKeywordsSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingKeywords;

    return it;
  }

  /** Will return an Iterator returning AuthUser objects. Preferences will
   * be attached - user objects will also be attached.
   */
  public Iterator getAuthUsers() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getAuthUsersSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingAuthUsers;

    return it;
  }

  /** Will return an Iterator returning EventVO objects.
   * All relevent objects, keywords, locations, sponsors, creators will
   * be attached.
   */
  public Iterator getEvents() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getEventsSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingEvents;

    return it;
  }

  /** Will return an Iterator returning EventRefVO objects.
   */
  public Iterator getEventRefs() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getEventRefsSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingEventRefs;

    return it;
  }

  /** Will return an Iterator returning AdminGroupVO objects.
   */
  public Iterator getAdminGroups() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getAdminGroupsSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingAdminGroups;

    return it;
  }

  /** Will return an Iterator returning DbLastmodVO objects.
   */
  public Iterator getDbLastmods() throws Throwable {
    PreparedStatement ps = conn.prepareStatement(getDbLastmodsSql[dbType]);

    JdbcDumpIterator it = new JdbcDumpIterator();
    it.rs = ps.executeQuery();
    it.type = gettingDbLastmods;

    return it;
  }

  public class JdbcDumpIterator implements Iterator {
    ResultSet rs;
    int type;

    /** True if hasNext has been called without an intervening next call.
     * reset to false when next called.
     *
     * If next() called without a call to hasNext() this will be false so
     * we should call the jdbc.next() method.
     */
    boolean doneHasNext;
    boolean hasNextResult;

    public boolean hasNext() {
      if (doneHasNext) {
        return hasNextResult;
      }

      try {
        hasNextResult = rs.next();
      } catch (Throwable t) {
        throw new RuntimeException(t);
      }

      doneHasNext = true;
      return hasNextResult;
    }

    public Object next() {
      if (!doneHasNext) {
        if (!hasNext()) {
          return null;
        }
      }

      doneHasNext = false;

      /** Positioned at the next result. Call the object specific method.
       */

      try {
        switch (type) {
          case gettingFilters:
            return getFilter();
          case gettingUsers:
            return getUser();
          case gettingLocations:
            return getLocation();
          case gettingSponsors:
            return getSponsor();
          case gettingKeywords:
            return getKeyword();
          case gettingEvents:
            return getEvent();
          case gettingEventRefs:
            return getEventRef();
          case gettingAuthUsers:
            return getAuthUser();
          case gettingAdminGroups:
            return getAdminGroup();
          case gettingDbLastmods:
            return getDbLastmod();
          default:
            throw new RuntimeException("Bad type: " + type);
        }
      } catch (Throwable t) {
        throw new RuntimeException(t);
      }
    }

    public void remove() {
      throw new RuntimeException("Remove not implemented");
    }

    private FilterVO getFilter() throws Throwable {
      FilterVO f;

      int id = rs.getInt("calendarid");
      String ftype = rs.getString("type");
      int refn = rs.getInt("ref_num");

      if (ftype.equals("A")) {
        f = new AndFilterVO();
      } else if (ftype.equals("K")) {
        f = new KeywordFilterVO();

        if (refn > 0) {
          KeywordVO k = new KeywordVO();
          k.setId(refn);

          ((KeywordFilterVO)f).setKeyword(k);
        }
      } else if (ftype.equals("C")) {
        f = new CreatorFilterVO();

        if (refn > 0) {
          UserVO u = new UserVO();
          u.setId(refn);

          ((CreatorFilterVO)f).setCreator(u);
        }
      } else if (ftype.equals("L")) {
        f = new LocationFilterVO();

        if (refn > 0) {
          LocationVO l = new LocationVO();
          l.setId(refn);

          ((LocationFilterVO)f).setLocation(l);
        }
      } else if (ftype.equals("O")) {
        f = new OrFilterVO();
      } else if (ftype.equals("S")) {
        f = new SponsorFilterVO();

        if (refn > 0) {
          SponsorVO sp = new SponsorVO();
          sp.setId(refn);

          ((SponsorFilterVO)f).setSponsor(sp);
        }
      } else if (ftype.equals("N")) {
        f = new NotFilterVO();

        if (refn > 0) {
          FilterVO fn = new FilterVO();
          fn.setId(refn);

          ((NotFilterVO)f).setFilter(fn);
        }
      } else if (ftype.equals("a")) {
        f = new AliasFilterVO();

        if (refn > 0) {
          FilterVO fn = new FilterVO();
          fn.setId(refn);

          ((AliasFilterVO)f).setTarget(fn);
        }
      } else {
        throw new Exception("Unimplemented filter " + ftype);
      }

      f.setId(id);
      f.setName(rs.getString("name"));
//      f.setTitle(rs.getString("title"));
//      f.setDescription(rs.getString("description"));

      filters.put(new Integer(f.getId()), f);

      int pkey = rs.getInt("parent");

      if (pkey != 0) {
        FilterVO pf = new FilterVO();

        pf.setId(pkey);
        f.setParent(pf);
      }

      return f;
    }

    private UserVO getUser() throws Throwable {
      UserVO u = new UserVO();

      u.setId(rs.getInt("userid"));
      u.setAccount(rs.getString("username"));
      u.setCreated(rs.getTimestamp("created"));
      u.setLogon(rs.getTimestamp("last_logon"));
      u.setLastAccess(rs.getTimestamp("last_access"));
      u.setLastModify(rs.getTimestamp("last_modify"));

      PreparedStatement ps = null;
      ResultSet ars = null;

      try {
        ps = conn.prepareStatement(getSubscriptionsSql[dbType]);

        ps.setInt(1, u.getId());

        ars = ps.executeQuery();

        Vector subs = new Vector();

        while (ars.next()) {
          FilterVO f = new FilterVO();

          f.setId(ars.getInt("calendarid"));
          subs.addElement(f);
        }

        u.setSubscriptions(subs);
      } finally {
        if (ars != null) {
          ars.close();
          ars = null;
        }
      }

      usersById.put(new Integer(u.getId()), u);
      usersByAccount.put(u.getAccount(), u);

      return u;
    }

    private LocationVO getLocation() throws Throwable {
      LocationVO l = new LocationVO();

      l.setId(rs.getInt("locationid"));
      l.setAddress(rs.getString("address"));
      l.setSubaddress(rs.getString("subaddress"));
      l.setLink(rs.getString("link"));

      String account = rs.getString("creator");
      l.setCreator(findUserByName(account));

      if (l.getCreator() == null) {
        System.err.println("Location owner " + account +
                           " does not exist for location " +
                           l.getId());
      }

      l.setIsPublic(rs.getString("publicf").equals("T"));

      return l;
    }

    private SponsorVO getSponsor() throws Throwable {
      SponsorVO s = new SponsorVO();

      s.setId(rs.getInt("sponsorid"));
      s.setName(rs.getString("name"));
      s.setPhone(rs.getString("phone"));
      s.setEmail(rs.getString("email"));
      s.setLink(rs.getString("link"));

      String account = rs.getString("creator");
      s.setCreator(findUserByName(account));

      if (s.getCreator() == null) {
        System.err.println("Sponsor owner " + account +
                           " does not exist for sponsor " +
                           s.getId());
      }

      s.setIsPublic(rs.getString("publicf").equals("T"));

      return s;
    }

    private KeywordVO getKeyword() throws Throwable {
      KeywordVO k = new KeywordVO();

      k.setId(rs.getInt("keywordid"));
      k.setWord(rs.getString("word"));
      k.setTitle(rs.getString("longdesc"));
      k.setDescription(rs.getString("longdesc1"));

      String account = rs.getString("creator");
      k.setCreator(findUserByName(account));

      if (k.getCreator() == null) {
        System.err.println("Keyword owner " + account +
                           " does not exist for keyword " +
                           k.getId());
      }

      k.setIsPublic(rs.getString("publicf").equals("T"));

      PreparedStatement ps = null;
      ResultSet ars = null;

      try {
        ps = conn.prepareStatement(getKeywordAttrsSql[dbType]);
        ps.setInt(1, k.getId());

        ars = ps.executeQuery();

        HashSet s = new HashSet();

        while (ars.next()) {
          KeywordAttrVO ka = new KeywordAttrVO();

          ka.setId(ars.getInt("kaid"));
          ka.setKeyword(k);
          ka.setName(ars.getString("kaname"));
          ka.setVal(ars.getString("kaval"));

          s.add(ka);
        }

        k.setAttributes(s);
      } finally {
        if (ars != null) {
          ars.close();
          ars = null;
        }
      }

      return k;
    }

    private AuthUserVO getAuthUser() throws Throwable {
      AuthUserVO au = new AuthUserVO();

      int id = rs.getInt("userid");
      au.setUser((UserVO)usersById.get(new Integer(id)));

      String autype = rs.getString("usertype");
      int type;

      if (autype.equals("A")) {
        type = UserAuth.superUser;
      } else if (autype.equals("C")) {
        type = UserAuth.contentAdminUser;
      } else if (autype.equals("a")) {
        type = UserAuth.alertUser;
      } else if (autype.equals("P")) {
        type = UserAuth.publicEventUser;
      } else {
        type = UserAuth.noPrivileges;
      }

      au.setUsertype(type);
      au.setLastname(rs.getString("lastname"));
      au.setFirstname(rs.getString("firstname"));
      au.setPhone(rs.getString("phone"));
      au.setEmail(rs.getString("email"));
      au.setDept(rs.getString("department"));

      if (au.getUser() == null) {
        return au;
      }

      PreparedStatement ps = null;
      ResultSet ars = null;

      try {
        ps = conn.prepareStatement(getAuthPrefsSql[dbType]);
        ps.setInt(1, au.getUser().getId());

        ars = ps.executeQuery();

        if (!ars.next()) {
          return au;
        }

        AuthUserPrefsVO aup = new AuthUserPrefsVO();

        aup.setId(au.getUser().getId());
        aup.setAutoAddKeywords(ars.getString("autoaddKeywords").equals("T"));
        aup.setAutoAddLocations(ars.getString("autoaddLocations").equals("T"));
        aup.setAutoAddSponsors(ars.getString("autoaddSponsors").equals("T"));

        ars.close();
        ars = null;

        if (aup.getAutoAddKeywords()) {
          ps = conn.prepareStatement(getAuthPrefKeywordsSql[dbType]);
          ps.setInt(1, au.getUser().getId());

          ars = ps.executeQuery();
          HashSet s = new HashSet();

          while (ars.next()) {
            KeywordVO k = new KeywordVO();

            k.setId(ars.getInt("keywordid"));
            s.add(k);
          }

          aup.setPreferredKeywords(s);
          ars.close();
          ars = null;
        }

        if (aup.getAutoAddLocations()) {
          ps = conn.prepareStatement(getAuthPrefLocationsSql[dbType]);
          ps.setInt(1, au.getUser().getId());

          ars = ps.executeQuery();
          HashSet s = new HashSet();

          while (ars.next()) {
            LocationVO l = new LocationVO();

            l.setId(ars.getInt("locationid"));
            s.add(l);
          }

          aup.setPreferredLocations(s);
          ars.close();
          ars = null;
        }

        if (aup.getAutoAddSponsors()) {
          ps = conn.prepareStatement(getAuthPrefSponsorsSql[dbType]);
          ps.setInt(1, au.getUser().getId());

          ars = ps.executeQuery();
          HashSet s = new HashSet();

          while (ars.next()) {
            SponsorVO sp = new SponsorVO();

            sp.setId(ars.getInt("sponsorid"));
            s.add(sp);
          }

          aup.setPreferredSponsors(s);
          ars.close();
          ars = null;
        }

        au.setPrefs(aup);
      } finally {
        if (ars != null) {
          ars.close();
          ars = null;
        }
      }

      return au;
    }

    private EventVO getEvent() throws Throwable {
      EventVO e = new EventVO();

      e.setId(rs.getInt("eventid"));
      String account = rs.getString("creator");
      e.setCreator(findUserByName(account));

      if (e.getCreator() == null) {
        System.err.println("Event owner " + account +
                           " does not exist for event " +
                           e.getId());
      }

      e.setPublic(rs.getString("publicf").equals("T"));
      e.setLastmod(rs.getTimestamp("lastmod"));
      e.setSeq(rs.getInt("seq"));
      e.setCreated(rs.getTimestamp("created"));
      e.setStartdate(rs.getDate("startdate"));
      e.setStarttime(rs.getTime("starttime"));
      e.setEnddate(rs.getDate("enddate"));
      e.setEndtime(rs.getTime("endtime"));
      e.setShortdesc(rs.getString("shortdesc"));
      e.setLongdesc(rs.getString("longdesc"));

      String longdesc1 = rs.getString("longdesc1");
      if ((longdesc1 != null) && (longdesc1.length() > 0)) {
        String longdesc = e.getLongdesc();

        if (longdesc == null) {
          e.setLongdesc(longdesc1);
        } else {
          e.setLongdesc(longdesc + longdesc1);
        }
      }

      e.setLink(rs.getString("link"));

      int id = rs.getInt("locationid");
      if (id != 0) {
        LocationVO l = new LocationVO();
        l.setId(id);
        e.setLocation(l);
      }

      id = rs.getInt("sponsorid");
      if (id != 0) {
        SponsorVO s = new SponsorVO();
        s.setId(id);
        e.setSponsor(s);
      }

      String recs = rs.getString("recurring_status");
      if ((recs == null) || (recs.length() == 0)) {
//        e.setRecurringStatus('N');
      } else {
        char recur = recs.charAt(0);

//        e.setRecurringStatus(recur);
      }

      e.setCost(rs.getString("cost"));

      PreparedStatement ps = null;
      ResultSet ars = null;

      try {
        ps = conn.prepareStatement(getEventKeywordsSql[dbType]);
        ps.setInt(1, e.getId());

        ars = ps.executeQuery();

        Vector v = new Vector();

        while (ars.next()) {
          KeywordVO k = new KeywordVO();

          k.setId(ars.getInt("keywordid"));

          v.addElement(k);
        }

        e.setKeywords(v);
      } finally {
        if (ars != null) {
          ars.close();
          ars = null;
        }
      }

      events.put(new Integer(e.getId()), e);

      return e;
    }

    private EventRefVO getEventRef() throws Throwable {
      EventRefVO e = new EventRefVO();

      e.setEventid(rs.getInt("eventid"));
      if (events.get(new Integer(e.getEventid())) == null) {
        System.err.println("Event  " + e.getEventid() +
                           " does not exist for event ref (" +
                           e.getEventid() + ", " + e.getUser());
      }

      int userid = rs.getInt("userid");

      UserVO u = (UserVO)usersById.get(new Integer(userid));
      e.setUser(u);

      if (u == null) {
        System.err.println("Event ref owner " + e.getUser() +
                           " does not exist for event ref (" +
                           e.getEventid() + ", " + e.getUser());
      }

      String purpose = rs.getString("purpose");
      if ((purpose == null) || (purpose.length() == 0)) {
        e.setPurpose('E');
      } else {
        e.setPurpose(purpose.charAt(0));
      }

      return e;
    }

    private AdminGroupVO getAdminGroup() throws Throwable {
      AdminGroupVO ag = new AdminGroupVO();

//      k.setId(rs.getInt("keywordid"));
      ag.setName(rs.getString("groupname"));
      ag.setDescription(rs.getString("description"));

      String account = rs.getString("groupOwner");
      ag.setGroupOwner(findUserByName(account));

      if (ag.getGroupOwner() == null) {
        System.err.println("Group owner " + account +
                           " does not exist for " +
                           ag.getName());
      }

      account = rs.getString("ownerid");
      ag.setOwner(findUserByName(account));

      if (ag.getOwner() == null) {
        System.err.println("Event owner " + account +
                           " does not exist for " +
                           ag.getName());
      }

      PreparedStatement ps = null;
      ResultSet ars = null;

      try {
        ps = conn.prepareStatement(getAdminGroupMembersSql[dbType]);
        ps.setString(1, ag.getName());

        ars = ps.executeQuery();

        HashSet s = new HashSet();

        while (ars.next()) {
          String mbrname = ars.getString("userid");
          UserVO u = findUserByName(mbrname);

          if (u == null) {
            System.err.println("Group member " + mbrname +
                               " does not exist for group " +
                               ag.getName());
          } else {
            s.add(u);
          }
        }

        ag.setGroupMembers(s);
      } finally {
        if (ars != null) {
          ars.close();
          ars = null;
        }
      }

      return ag;
    }

    private DumpDefs.DbLastmodVO getDbLastmod() throws Throwable {
      DumpDefs.DbLastmodVO lm = new DumpDefs.DbLastmodVO();

      lm.setDbname(rs.getString("name"));
      lm.setLastmod(rs.getTimestamp("lastmod"));

      return lm;
    }

    private UserVO findUserByName(String name) throws Throwable {
      UserVO u = (UserVO)usersByAccount.get(name);

      if (u != null) {
        return u;
      }

      PreparedStatement ps = null;
      ResultSet ars = null;

      try {
        ps = conn.prepareStatement(
              "select * from users where username='" + name + "'");

        ars = ps.executeQuery();

        if (!ars.next()) {
          return null;
        }

        u = new UserVO();

        u.setId(ars.getInt("userid"));

        return u;
      } finally {
        if (ars != null) {
          ars.close();
          ars = null;
        }
      }
    }

    private UserVO findUserById(int id) throws Throwable {
      UserVO u = (UserVO)usersById.get(new Integer(id));

      if (u != null) {
        return u;
      }

      PreparedStatement ps = null;
      ResultSet ars = null;

      try {
        ps = conn.prepareStatement(
              "select * from users where userid=" + id);

        ars = ps.executeQuery();

        if (!ars.next()) {
          return null;
        }

        u = new UserVO();

        u.setId(id);

        return u;
      } finally {
        if (ars != null) {
          ars.close();
          ars = null;
        }
      }
    }
  }
}

