package edu.rpi.cct.uwcal.tools.dump;

import edu.washington.cac.calfacade.shared.AdminGroupVO;
import edu.washington.cac.calfacade.shared.AuthUserVO;
import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.EventRefVO;
import edu.washington.cac.calfacade.shared.filter.FilterVO;
import edu.washington.cac.calfacade.shared.KeywordVO;
import edu.washington.cac.calfacade.shared.LocationVO;
import edu.washington.cac.calfacade.shared.SponsorVO;
import edu.washington.cac.calfacade.shared.UserVO;

/** Interface which defines the database functions needed to restore the
 * calendar database. The methods need to be called in the order defined
 * below.
 *
 * @author Mike Douglass   douglm@rpi.edu
 * @version 1.0
 */
public interface RestoreIntf {
  /** Call after any init phase
   */
  public void open() throws Throwable;

  /** Call at end of restoring to finish up. Will restore any cached values.
   */
  public void close() throws Throwable;

  /** Restore users NOT subscriptions
   */
  public void restoreUser(UserVO o) throws Throwable;

  /** Restore an admin group - though not the user entries nor
   * the authuser entries.
   */
  public void restoreAdminGroup(AdminGroupVO o) throws Throwable;

  /** Restore an auth user and preferences
   */
  public void restoreAuthUser(AuthUserVO o) throws Throwable;

  /** Restore an event and event keyword entries
   */
  public void restoreEvent(EventVO o) throws Throwable;

  /** Restore event ref
   */
  public void restoreEventRef(EventRefVO o) throws Throwable;

  /** Restore keyword and attributes
   */
  public void restoreKeyword(KeywordVO o) throws Throwable;

  /** Restore location
   */
  public void restoreLocation(LocationVO o) throws Throwable;

  /** Restore sponsor
   */
  public void restoreSponsor(SponsorVO o) throws Throwable;

  /** Restore filter
   */
  public void restoreFilter(FilterVO o) throws Throwable;

  /** Subscribe user to filter.
   */
  public void subscribe(UserVO user, FilterVO f) throws Throwable;

  public void restoreDbLastmod(DumpDefs.DbLastmodVO o) throws Throwable;
}

